/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit;

import java.util.*;

/**
 * The superclass for all test contexts that are capable of having
 * other nested (child) test contexts.
 *
 * @author    Simon Brown
 */
public class DefaultTestContextContainer extends DefaultTestContext implements TestContextContainer {

  /** the collection of text contexts that are nested underneath this one */
  private Collection children = new ArrayList();

  /** the object responsible for managing test contexts */
  private transient TestContextManager manager;

  /**
   * Default, no args constructor.
   */
  public DefaultTestContextContainer() {
  }

  /**
   * Creates a new test context with the specified name.
   *
   * @param name    the name of this context
   */
  public DefaultTestContextContainer(String name) {
    super(name);
  }

  /**
   * Sets the object responsible for managing this and related test contexts.
   *
   * @param manager   a TestContextManager instance
   */
  public void setManager(TestContextManager manager) {
    this.manager = manager;
  }

  /**
   * Gets the object responsible for managing this and related test contexts.
   *
   * @return  a TestContextManager instance
   */
  public TestContextManager getManager() {
    if (this.manager != null) {
      return this.manager;
    } else {
      return getParent().getManager();
    }
  }

  /**
   * Gets the result of the tests that were run within this test context.
   *
   * @return  true if the tests were successful, false otherwise
   */
  public boolean getResult() {
    if (!hasChildren()) {
      return super.getResult();
    } else {
      boolean success = true;
      TestContext ctx;
      Iterator it = children.iterator();
      while (it.hasNext()) {
        ctx = (TestContext)it.next();
        success &= ctx.getResult();
      }

      if (!super.getStatus().equals(TestContext.UNDEFINED)) {
        // return (child have passed && this context has passed)
        return success && super.getResult();
      } else {
        return success;
      }
    }
  }

  /**
   * Gets the status of this test context (i.e. whether it passed,
   * failed, has warnings, etc).
   *
   * @return  a TestContextStatus instance representing the status of
   *          this test context
   */
  public TestContextStatus getStatus() {
    if (!hasChildren()) {
      return super.getStatus();
    }

    // in this set we'll collect all of the child statuses, from which
    // we can then pick the most severe and return it
    Set statuses = new HashSet();
    if (!super.getStatus().equals(TestContext.UNDEFINED)) {
      statuses.add(super.getStatus());
    }

    TestContext ctx;
    Iterator it = children.iterator();
    while (it.hasNext()) {
      ctx = (TestContext)it.next();
      statuses.add(ctx.getStatus());
    }

    if (statuses.contains(TestContext.ERROR)) {
      return TestContext.ERROR;
    } else if (statuses.contains(TestContext.FAIL)) {
      return TestContext.FAIL;
    } else if (statuses.contains(TestContext.WARNING)) {
      if (TagUnitProperties.getInstance().getIgnoreWarnings()) {
        return TestContext.PASS;
      } else {
        return TestContext.WARNING;
      }
    } else {
      return TestContext.PASS;
    }
  }

  /**
   * Adds a child test context.
   *
   * @param ctx   the new nested/child TestContext instance
   */
  public void add(TestContext ctx) {
    if (!children.contains(ctx)) {
      children.add(ctx);
    }
    ctx.setParent(this);

    // and set the type of the context the same as this one
    ctx.setType(getType());
  }

  /**
   * Gets a Collection containing all of the child test contexts.
   *
   * @return  a Collection of TestContext instances
   */
  public Collection getChildren() {
    return Collections.unmodifiableCollection(children);
  }

  public boolean hasChildren() {
    return !children.isEmpty();
  }

  public int getNumberOfTests() {
    return getNumberOfPasses() + getNumberOfWarnings() + getNumberOfFailures();
  }

  /**
   * Gets the number of tests that this context represents
   * with the specified status.
   *
   * @return  the number of tests as an integer
   */
  public int getNumberOfTests(TestContextStatus status) {

    if (!hasChildren()) {
      return status.equals(getStatus()) ? 1 : 0;
    }

    int count = 0;
    if (!super.getStatus().equals(TestContext.UNDEFINED)) {
      count = super.getStatus().equals(status) ? 1 : 0;
    }

    Iterator it = children.iterator();
    while (it.hasNext()) {
      Object o = it.next();
      if (o instanceof TestContextContainer) {
        TestContextContainer tcc = (TestContextContainer)o;
        if (tcc.hasChildren()) {
          count += tcc.getNumberOfTests(status);
          ;
        } else if (tcc.getStatus().equals(status)) {
          count++;
        }
      } else if (o instanceof TestContext) {
        TestContext ctx = (TestContext)o;
        if (ctx.getStatus().equals(status)) {
          count++;
        }
      }
    }

    return count;
  }

  /**
   * Gets the number of passing tests that this context represents.
   *
   * @return  the number of tests as an integer
   */
  public int getNumberOfPasses() {
    return getNumberOfTests(TestContext.PASS);
  }

  /**
   * Gets the number of tests with warnings that this context represents.
   *
   * @return  the number of tests as an integer
   */
  public int getNumberOfWarnings() {
    return getNumberOfTests(TestContext.WARNING);
  }

  /**
   * Gets the number of failing tests that this context represents.
   *
   * @return  the number of tests as an integer
   */
  public int getNumberOfFailures() {
    return getNumberOfTests(TestContext.FAIL);
  }

  /**
   * Gets the number of tests in error that this context represents.
   *
   * @return  the number of tests as an integer
   */
  public int getNumberOfErrors() {
    return getNumberOfTests(TestContext.ERROR);
  }

  /**
   * For debug purposes only, dumps this context to stdout.
   */
  public void dump() {
    dump(this, 0);
  }

  /**
   * For debug purposes only, dumps this context to stdout.
   *
   * @param ctx   the TestContext instance to be dumped
   * @param level the level at which the TestContext is at (in the hierarchy)
   */
  private void dump(TestContext ctx, int level) {
    // pretty formatting of output.
    String spaces = "           ";
    String nudge = (level < spaces.length()) ? spaces.substring(0, level) : spaces;
    System.out.println(nudge + "(" + level + ") " + ctx.getName() + " (" + ctx.getResult() + " : " + ctx.getStatus().getName() + ")");

    if (ctx instanceof TestContextContainer) {
      Iterator it = ((TestContextContainer)ctx).getChildren().iterator();
      while (it.hasNext()) {
        TestContext tc = (TestContext)it.next();
        dump(tc, level + 1);
      }
    }
  }

}