/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit;

import java.util.Map;
import java.util.TreeMap;

import org.tagunit.taginfo.TagInfo;
import org.tagunit.taginfo.TagLibraryInfo;

/**
 * Represents the top level context in which all tests for a particular
 * tag library are executed.
 *
 * @author    Simon Brown
 */
public class TagLibraryTestContext extends DefaultTestContextContainer {

  /** a reference to the information about the tag library being tested */
  private TagLibraryInfo tagLibraryInfo;

  /** the collection of test contexts for the tags in the associated taglib */
  private Map tagTestContexts = new TreeMap();

  /**
   * Default, no args constructor.
   */
  public TagLibraryTestContext() {
  }

  /**
   * Default, no args constructor.
   *
   * @param name    the name for this context
   */
  public TagLibraryTestContext(String name) {
    super(name);
  }

  /**
   * Default, no args constructor.
   *
   * @param tagLibraryInfo    a TagLibraryInfo instance
   */
  public TagLibraryTestContext(TagLibraryInfo tagLibraryInfo) {
    setTagLibraryInfo(tagLibraryInfo);
  }

  /**
   * Gets the information about the tag library being tested.
   *
   * @return  a TagLibraryInfo instance
   */
  public TagLibraryInfo getTagLibraryInfo() {
    return this.tagLibraryInfo;
  }

  /**
   * Sets the information about the tag library being tested.
   *
   * @param info    a TagLibraryInfo instance
   */
  public void setTagLibraryInfo(TagLibraryInfo info) {
    this.tagLibraryInfo = info;
    setName("Tests for " + tagLibraryInfo.getShortName() + " (v" + tagLibraryInfo.getVersion() + ")");
  }

  /**
   * Adds a child test context.
   *
   * @param ctx   the new nested/child TestContext instance
   */
  public void add(TestContext ctx) {
    super.add(ctx);

    if (ctx instanceof TagTestContext) {
      tagTestContexts.put(((TagTestContext)ctx).getTagInfo().getName(), ctx);
    }
  }

  /**
   * Gets the TagTestContext for the named tag, creating one if necessary.
   *
   * @param tagInfo   a TagInfo object representing the tag
   * @return    a TagTestContext for the specified tag
   */
  public TagTestContext getTagTestContext(TagInfo tagInfo) {
    TagTestContext ctx = (TagTestContext)tagTestContexts.get(tagInfo.getName());

    if (ctx == null) {
      ctx = new TagTestContext(tagInfo.getName());
      ctx.setTagInfo(tagInfo);
      add(ctx);
    }

    return ctx;
  }

}