/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.ant;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.util.DOMElementWriter;

import org.tagunit.TestContextContainer;
import org.tagunit.TestContext;
import org.tagunit.taginfo.TagInfo;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.xml.parsers.*;
import java.io.*;

public class XmlFormatter extends AbstractFormatter {

    private Document document;

    public XmlFormatter() {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        try {
            DocumentBuilder builder = dbf.newDocumentBuilder();
            document = builder.newDocument();
        } catch (ParserConfigurationException e) {
            e.printStackTrace();
        }
    }

    public void displayHeader(TestContextContainer context) {
        Element root = document.createElement("testsuite");

        root.setAttribute("name", context.getName());
        root.setAttribute("tests", new Integer(context.getNumberOfTests()).toString());
        root.setAttribute("time", new Long(context.getTime()).toString());
        root.setAttribute("errors", new Integer(context.getNumberOfErrors()).toString());
        root.setAttribute("failures", new Integer(context.getNumberOfFailures()).toString());
        Element child = document.createElement("properties");

        root.appendChild(child);

        document.appendChild(root);

    }

    public void displaySummary(TestContextContainer context) {

        // todo: add the standard our and error

        Element sout = document.createElement("system-out");
        Element serr = document.createElement("system-err");

        document.getDocumentElement().appendChild(sout);
        document.getDocumentElement().appendChild(serr);
        
        try {
            Writer writer = new OutputStreamWriter( outStream, "UTF8" );
            writer.write("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");
            new DOMElementWriter().write(document.getDocumentElement(), writer, 0, "  ");
            writer.flush();
        } catch (IOException e) {
            throw new BuildException( "Failed to write to output stream.", e );
        }
    }

    public void displayTest(TestContext context) {
        Element testcase = addTestCase(context);

        if (context.getStatus().equals(TestContext.FAIL)
                       || context.getStatus().equals(TestContext.WARNING)
                        || context.getStatus().equals(TestContext.ERROR)) {
            addFailure(context, testcase);
        }

        document.getDocumentElement().appendChild(testcase);

    }

    private void addFailure(TestContext context, Element testcase) {
        // add the failure/error element
        Element child = null;
        String message = context.getMessage();
        String summary = "";
        try {
            summary = new BufferedReader(new StringReader(message)).readLine();
        } catch (IOException e) {
        }

        if (context.getStatus().equals( TestContext.FAIL ) ) {
            child = document.createElement("failure");
        } else if (context.getStatus().equals( TestContext.ERROR ) ) {
            child = document.createElement("error");
        } else if (context.getStatus().equals( TestContext.WARNING ) ) {
            child = document.createElement("warning");
        }
        
        if (child != null) {
            child.setAttribute("message", summary);
            child.setAttribute("type", "org.tagunit.AssertionFailedError");
            child.setNodeValue(message);
            testcase.appendChild(child);
        }
    }

    private Element addTestCase(TestContext context) {
        String name;
        name = getTestName(context);

        Element testcase = document.createElement("testcase");

        testcase.setAttribute("name", name);
        testcase.setAttribute("time", new Long(context.getTime()).toString());
        return testcase;
    }

    private String getTestName(TestContext context) {
        String name;
        TagInfo tag = getTag(context);

        if (tag != null) {
            name = tag.getName() + " : " + context.getName();
        } else {
            name = context.getName();
        }
        return name;
    }
}
