/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.controller;

import java.io.*;
import java.util.*;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.tagunit.*;
import org.tagunit.taginfo.*;
import org.tagunit.test.TagLibraryTestPackage;
import org.tagunit.util.ExceptionUtils;

/**
 * Responsible for initiating the tests.
 *
 * @author    Simon Brown
 */
public class RunTestsAction extends Action {

  /**
   * Peforms the processing associated with this action.
   *
   * @param request  The HttpServletRequest instance.
   * @param response   The HttpServletResponse instance.
   * @return       The name of the next view
   */
  public String process(HttpServletRequest request,
                        HttpServletResponse response)
      throws ServletException {

    // setup a new root test context and a manager to manage the tests
    TestContextContainer rootTestContext = new DefaultTestContextContainer("TagUnit tests run at " + new Date());
    TestContextManager manager = new TestContextManager(rootTestContext);

    // put the manager into the request as the tests will make use of this to
    // add new nested test contexts during execution
    request.setAttribute(Constants.TEST_CONTEXT_MANAGER, manager);

    // are we testing a specific tag?
    String tag = request.getParameter("tag");
    if (tag != null && tag.length() > 0) {
      request.setAttribute(Constants.TAG_TO_BE_TESTED, tag);
    }

    String uri = request.getParameter("uri");
    if (uri != null && uri.length() > 0) {
      // run the tests located at the specified URI
      runTests(uri, tag, request, response, rootTestContext);
    } else {
      // run the automatic tag tests for all tag libraries within the web application
      runTestsOnTlds(rootTestContext, "/WEB-INF/");
      runTestsOnTlds(rootTestContext, "/WEB-INF/tlds/");
      runTestsOnJars(rootTestContext);
    }

    request.getSession().setAttribute(Constants.TEST_CONTEXT, rootTestContext);

    // the automatic (Ant) client sends over a "client=auto" parameter
    String client = request.getParameter("client");
    if (client != null && client.equalsIgnoreCase("auto")) {
      return "servlet/SerializeResults";
    } else {
      return "servlet/ViewResults";
    }
  }

  /**
   * Runs the tests that are located at the specified URI.
   *
   * @param uri   the URI representing the starting point for the tests
   * @param tag   the name of the tag to run the tests for
   * @param request   the current ServletRequest
   * @param response  the current ServletResponse
   * @param rootTestContext   the TestContext under which the tests should run
   */
  private void runTests(String uri, String tag, HttpServletRequest request, HttpServletResponse response, TestContextContainer rootTestContext) {
    try {
      RequestDispatcher dispatcher = request.getRequestDispatcher(uri);
      dispatcher.include(
        new TestHttpServletRequestWrapper(request),
        new TestHttpServletResponseWrapper(response));
    } catch (Throwable e) {
      e.printStackTrace();
      rootTestContext.setStatus(TestContext.ERROR);
      rootTestContext.setMessage(ExceptionUtils.getStackTraceAsString(e));
    }
  }

  /**
   * Runs the automatic tests for all tag libraries in the current web application.
   *
   * @param rootTestContext   the TestContext under which the tests should run
   */
  private void runTestsOnTlds(TestContextContainer rootTestContext, String path) {
    // find tag libraries at WEB-INF/*.tld directory
    Iterator it = findTLDs(path).iterator();
    while (it.hasNext()) {
      TagLibraryTestContext tltc = null;

      try {
        String uri = (String)it.next();
        tltc = new TagLibraryTestContext("Tests for tag library at " + uri);
        rootTestContext.add(tltc);

        InputStream in = getServletContext().getResourceAsStream(uri);
        TagLibraryInfo tagLibraryInfo = new TagLibraryInfoFactory().getTagLibraryInfo(in);
        tltc.setTagLibraryInfo(tagLibraryInfo);

        rootTestContext.getManager().startTestContext(tltc);
        TagLibraryTestPackage tests = new TagLibraryTestPackage(tagLibraryInfo, null, null, null);
        tests.executeTest(tltc);
        rootTestContext.getManager().endTestContext(tltc);
      } catch (TagLibraryNotSupportedException tlnse) {
        tltc.setStatus(TestContext.FAIL);
        tltc.setMessage(tlnse.getMessage());
      } catch (TagLibraryInfoParseException tlipe) {
        tltc.setStatus(TestContext.FAIL);
        tltc.setMessage(tlipe.getMessage());
      } catch (Throwable e) {
        e.printStackTrace();
        rootTestContext.setStatus(TestContext.ERROR);
        rootTestContext.setMessage(ExceptionUtils.getStackTraceAsString(e));
      }
    }
  }

  /**
   * Given a URI, this method looks for files called "*.tld".
   *
   * @param path    a URI, relative from the root of a webapp
   * @return    a Collection of URIs pointing to TLD files
   */
  private Collection findTLDs(String path) {
    Collection resources = getServletContext().getResourcePaths(path);
    Collection result = new ArrayList();

    if (resources == null || resources.isEmpty()) {
      return result;
    }

    Iterator it = resources.iterator();
    String uri;

    // for each file in the directory, check that it matches the
    // filename filter
    while (it.hasNext()) {
      uri = (String)it.next();

      // find the last "/" character
      int index = uri.lastIndexOf('/');
      String s = uri.substring(index + 1, uri.length());
      if (s.endsWith(".tld")) {
        result.add(uri);
      }
    }

    return result;
  }

  /**
   * Runs the automatic tests for all tag libraries in the current web application.
   *
   * @param rootTestContext   the TestContext under which the tests should run
   */
  private void runTestsOnJars(TestContextContainer rootTestContext) {

    Iterator it = findJARs().iterator();
    while (it.hasNext()) {
      String uri = (String)it.next();
      TestContextContainer jarContext = new DefaultTestContextContainer("Tests for tag libraries at " + uri);

      Collection tagLibraries = loadTldsFromJar(jarContext, uri);
      TagLibraryTestContext tltc;
      TagLibraryInfo tagLibraryInfo;

      if (tagLibraries.isEmpty()) {
        continue;
      }

      rootTestContext.add(jarContext);
      rootTestContext.getManager().startTestContext(jarContext);

      Iterator jt = tagLibraries.iterator();
      while (jt.hasNext()) {
        tltc = (TagLibraryTestContext)jt.next();
        tagLibraryInfo = tltc.getTagLibraryInfo();

        if (tagLibraryInfo != null) {
          rootTestContext.getManager().startTestContext(tltc);
          TagLibraryTestPackage tests = new TagLibraryTestPackage(tagLibraryInfo, null, null, null);
          tests.executeTest(tltc);
          rootTestContext.getManager().endTestContext(tltc);
        }
      }

      rootTestContext.getManager().endTestContext(jarContext);
    }
  }

  /**
   * Looks for JAR files in the WEB-INF/lib directory.
   *
   * @return    a Collection of URIs pointing to JAR files
   */
  private Collection findJARs() {
    Collection resources = getServletContext().getResourcePaths("/WEB-INF/lib/");
    Collection result = new ArrayList();

    if (resources == null || resources.isEmpty()) {
      return result;
    }

    Iterator it = resources.iterator();
    String uri;

    // for each file in the directory, check that it matches the
    // filename filter
    while (it.hasNext()) {
      uri = (String)it.next();

      // find the last "/" character
      int index = uri.lastIndexOf('/');
      String s = uri.substring(index + 1, uri.length());
      if (s.endsWith(".jar")) {
        result.add(uri);
      }
    }

    return result;
  }

  /**
   * Loads a TLD file from a JAR file (within the META-INF directory).
   *
   * @return  a populated TagLibraryInfo instance representing the taglib,
   *          or null if a taglib couldn't be found
   */
  private Collection loadTldsFromJar(TestContextContainer jarContext, String uri) {

    Collection result = new ArrayList();

    File file = null;
    try {
      // we have the name of a JAR file, and the name of the TLD file within
      // that JAR file
      InputStream jarInputStream = getServletContext().getResourceAsStream(uri);

      // @todo  Optimize the reading and writing of the JAR file
      // this next section reads in the JAR file from the InputStream and
      // saves a copy to the temporary directory made available by the
      // servlet container
      // (there's not a simple way of reading in a file inside a JAR file
      // that is itself referenced by a URI) :-(
      BufferedInputStream bin = new BufferedInputStream(jarInputStream);
      File tmp = (File)getServletContext().getAttribute(Constants.CONTAINER_PROVIDED_TEMPORARY_DIRECTORY);
      // create a copy of the JAR file pointed to URI in the container temp directory
      // (the name of the JAR file is the same)
      file = new File(tmp, uri.substring(uri.lastIndexOf("/")+1, uri.length()));
      BufferedOutputStream bout = new BufferedOutputStream(new FileOutputStream(file));
      int b = bin.read();
      while (b != -1) {
        bout.write(b);
        b = bin.read();
      }
      bout.flush();
      bout.close();
      bin.close();

      // the copy of the JAR file is then used to get an InputStream to the
      // TLD file residing inside the JAR
      JarFile jarFile = new JarFile(file);
      Enumeration enum = jarFile.entries();
      while (enum.hasMoreElements()) {
        JarEntry entry = (JarEntry)enum.nextElement();
        if (entry.getName().startsWith("META-INF/") && entry.getName().endsWith(".tld")) {
          TagLibraryTestContext tltc = new TagLibraryTestContext("Tests for tag library at " + uri);
          jarContext.add(tltc);

          try {
            // and now try to parse the TLD file
            InputStream in = jarFile.getInputStream(entry);
            tltc.setTagLibraryInfo(new TagLibraryInfoFactory().getTagLibraryInfo(in));
            result.add(tltc);
          } catch (TagLibraryNotSupportedException tlnse) {
            tltc.setStatus(TestContext.FAIL);
            tltc.setMessage(tlnse.getMessage());
          } catch (TagLibraryInfoParseException tlipe) {
            tltc.setStatus(TestContext.FAIL);
            tltc.setMessage(tlipe.getMessage());
          }
        }
      }
    } catch (IOException ioe) {
      jarContext.setStatus(TestContext.ERROR);
      jarContext.setMessage(ioe.getMessage());
    }

    return result;
  }

}