/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.controller;

import java.util.*;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.tagunit.*;

/**
 * Filters the test results, ready for them to be displayed by a JSP view.
 *
 * @author    Simon Brown
 */
public class ViewResultsAction extends Action {

  /**
   * Peforms the processing associated with this action.
   *
   * @param request  The HttpServletRequest instance.
   * @param response   The HttpServletResponse instance.
   * @return       The name of the next view
   */
  public String process(HttpServletRequest request,
                        HttpServletResponse response)
      throws ServletException {

    // find the top level test context
    TestContextContainer ctx = (TestContextContainer)request.getSession().getAttribute(Constants.TEST_CONTEXT);

    // are we including the tests that passed when the information
    // is presented to the user?
    boolean showPasses = getBooleanParameter(request, "showPasses");
    boolean showWarnings = getBooleanParameter(request, "showWarnings");
    String tag = request.getParameter("tag");

    // filter/flatten the results and put them into the user's session
    Collection resultsToDisplay = filterResults(ctx, 0, showPasses, showWarnings, tag);

    // toggle on/off whether warnings are presented to the user
    String ignoreWarnings = request.getParameter("ignoreWarnings");
    if (ignoreWarnings != null) {
      if (ignoreWarnings.equalsIgnoreCase("true")) {
        TagUnitProperties.getInstance().setIgnoreWarnings(true);
      } else {
        TagUnitProperties.getInstance().setIgnoreWarnings(false);
      }
    }

    // put the objects into the request
    request.setAttribute(Constants.TEST_RESULTS, resultsToDisplay);
    request.setAttribute("testContext", ctx);

    // finally, redirect to the appropriate view component for presentation
    return "results.jsp";
  }

  /**
   * Helper method to get the boolean value of a request parameter.
   *
   * @param request   the ServletRequest to get the parameter from
   * @param name      the name of the parameter
   * @return          true if the parameter is present and has a value of true
   *                  (ignoring case), or false otherwise
   */
  private boolean getBooleanParameter(HttpServletRequest request, String name) {
    String s = request.getParameter(name);
    if (s == null || s.length() == 0) {
      return true;
    } else {
      return s.equalsIgnoreCase("true");
    }
  }

  /**
   * A recursive method to flattens an object graph of TestContext instances
   * into a list of TestResult instances. Each of these contains a reference
   * to the actual TestContext in addition to an integer representing the level
   * of the TestContext in the tree (zero for the root, one for its children, etc).
   *
   * @param rootTestContext   the TestContext to be "flattened"
   * @param level   the current level
   * @return  a Collection of TestResult instances
   */
  private Collection filterResults(TestContext rootTestContext, int level,
                                   boolean showPasses, boolean showWarnings, String tag) {

    // if we're not showing passes and everything passed - return nothing!
    if (!showPasses && rootTestContext.getStatus() == TestContext.PASS && level > 0) {
      return new ArrayList();
    } else if (!showWarnings && rootTestContext.getStatus() == TestContext.WARNING && level > 0) {
      return new ArrayList();
    }

    // add the specified Menu itself as the first element in the list
    List testResults = new ArrayList();
    testResults.add(new TestResult(rootTestContext, level));

    if (rootTestContext instanceof TestContextContainer) {
      if (tag != null && rootTestContext instanceof TagTestContext) {
        TagTestContext ttc = (TagTestContext)rootTestContext;
        if (!ttc.getTagInfo().getName().equals(tag)) {
          return testResults;
        }
      }

      TestContext testContext;
      Iterator it;

      // and now flatten the children (going one more level down)
      level++;
      it = ((TestContextContainer)rootTestContext).getChildren().iterator();
      while (it.hasNext()) {
        testContext = (TestContext)it.next();
        // flatten the next test context
        testResults.addAll(filterResults(testContext, level, showPasses, showWarnings, tag));
      }
    }

    return testResults;
  }

}