/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.tagext.test;

import java.io.*;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import org.tagunit.*;
import org.tagunit.taginfo.*;

/**
 * Locates a specified TLD file, which is subsequently used in some of the
 * tests and assertions within the framework.
 *
 * @author    Simon Brown
 */
public class TagLibraryDescriptorTag extends TestContextTag {

  /** represents the URI of the TLD file */
  private String uri;

  /** represents the name of the JAR file in which the TLD resides */
  private String jar;

  /**
   * Template method called during doStartTag(). Tags can override this to
   * perform any custom processing.
   *
   * @param testContext   the current test context container
   */
  protected void doStartTag(TestContextContainer testContext) {
    TagLibraryTestContext tagLibraryTestContext = (TagLibraryTestContext)testContext.getParent();

    // the purpose of this tag is to find the TLD file and load the information
    // that is stored within it
    try {
      TagLibraryInfo info;
      if (uri != null) {
        info = loadTldFromUri(testContext);
      } else {
        info = loadTldFromJar(testContext);
      }

      if (info != null) {
        tagLibraryTestContext.setTagLibraryInfo(info);
        testContext.setStatus(TestContext.PASS);
      }
    } catch (TagLibraryNotSupportedException tlnse) {
      testContext.setStatus(TestContext.FAIL);
      testContext.setMessage(tlnse.getMessage());
    } catch (TagLibraryInfoParseException tlipe) {
      testContext.setStatus(TestContext.FAIL);
      testContext.setMessage(tlipe.getMessage());
    } catch (Exception e) {
      doCatch(e);
    }
  }

  /**
   * Loads a TLD file from a URI.
   *
   * @param testContext   the current test context
   * @return  a populated TagLibraryInfo instance representing the taglib,
   *          or null if a taglib couldn't be found
   * @throws Exception    should anything go wrong
   */
  private TagLibraryInfo loadTldFromUri(TestContext testContext) throws Exception {
    // we have a direct URI to the TLD file so let's try to load it
    // (in other words, the TLD file sits underneath WEB-INF somewhere)
    testContext.setName("Tag library descriptor file at " + uri);
    InputStream in = pageContext.getServletContext().getResourceAsStream(uri);

    // the TLD file couldn't be found
    if (in == null) {
      testContext.setStatus(TestContext.FAIL);
      testContext.setMessage("A TLD at " + uri + " was not found");

      return null;
    }

    // and now try to parse the TLD file
    return new TagLibraryInfoFactory().getTagLibraryInfo(in);
  }

  /**
   * Loads a TLD file from a JAR file (within the META-INF directory).
   *
   * @param testContext   the current test context
   * @return  a populated TagLibraryInfo instance representing the taglib,
   *          or null if a taglib couldn't be found
   * @throws Exception    should anything go wrong
   */
  private TagLibraryInfo loadTldFromJar(TestContext testContext) throws Exception {
    // we have the name of a JAR file, and the name of the TLD file within
    // that JAR file
    testContext.setName("Tag library descriptor file (" + getName() + ") inside " + jar);

    InputStream jarInputStream = pageContext.getServletContext().getResourceAsStream("/WEB-INF/lib/" + jar);

    // the JAR file couldn't be loaded
    if (jarInputStream == null) {
      testContext.setStatus(TestContext.FAIL);
      testContext.setMessage("A JAR file at /WEB-INF/lib/" + jar + " was not found");

      return null;
    }

    // @todo  Optimize the reading and writing of the JAR file
    // this next section reads in the JAR file from the InputStream and
    // saves a copy to the temporary directory made available by the
    // servlet container
    // (there's not a simple way of reading in a file inside a JAR file
    // that is itself referenced by a URI) :-(
    BufferedInputStream bin = new BufferedInputStream(jarInputStream);
    File tmp = (File)pageContext.getServletContext().getAttribute(Constants.CONTAINER_PROVIDED_TEMPORARY_DIRECTORY);
    File file = new File(tmp, jar);
    BufferedOutputStream bout = new BufferedOutputStream(new FileOutputStream(file));
    int b = bin.read();
    while (b != -1) {
      bout.write(b);
      b = bin.read();
    }
    bout.flush();
    bout.close();
    bin.close();

    // the copy of the JAR file is then used to get an InputStream to the
    // TLD file residing inside the JAR
    JarFile jarFile = new JarFile(file);
    JarEntry entry = jarFile.getJarEntry("META-INF/" + getName());

    // check that the TLD file exists within the JAR file
    if (entry == null) {
      testContext.setStatus(TestContext.FAIL);
      testContext.setMessage("A TLD at /META-INF/" + getName() + " in " + jar + " was not found");

      return null;
    }

    // and now try to parse the TLD file
    InputStream in = jarFile.getInputStream(entry);
    return new TagLibraryInfoFactory().getTagLibraryInfo(in);
  }

  /**
   * Setter for the uri attribute.
   *
   * @param s   a String representing the URI
   */
  public void setUri(String s) {
    this.uri = s;
  }

  /**
   * Setter for the jar attribute.
   *
   * @param s   a String representing the name of the JAR file
   */
  public void setJar(String s) {
    this.jar = s;
  }

}