/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.tagext.test;

import java.io.IOException;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.jsp.tagext.TryCatchFinally;

import org.tagunit.*;
import org.tagunit.util.ExceptionUtils;

/**
 * The superclass of all test and assertion tags within the framework.
 *
 * @author    Simon Brown
 */
public abstract class TestContextTag extends TagSupport implements TryCatchFinally {

  /** the name of the test/assertion that is being performed */
  private String name;

  /** the test context for this tag and its contents */
  private TestContextContainer testContext;

  /** a reference to the Throwable thrown during execution */
  private Throwable throwable;

  /**
   * Creates the test context container for this tag.
   *
   * @return    a TestContextContainer instance
   */
  protected TestContextContainer createTestContext() {
    return new DefaultTestContextContainer(getName());
  }

  /**
   * Called when the starting tag is encountered.
   */
  public final int doStartTag() throws JspException {
    // set and reset any state held by this tag handler instance
    this.testContext = createTestContext();
    this.throwable = null;

    // check that the TestContextManager has been created for this test run
    if (getTestContextManager() == null) {
      try {
        JspWriter out = pageContext.getOut();
        if (out != null) {
          out.write("All tests should be run through the controller servlet - please see the documentation for more details.");
        }
      } catch (IOException e) {
      }

      throw new JspException("All tests should be run through the controller servlet - please see the documentation for more details.");
    }

    getTestContextManager().startTestContext(testContext);
    doStartTag(testContext);

    // let's just include/evaluate the body content of the tag
    return EVAL_BODY_INCLUDE;
  }

  /**
   * Template method called during doStartTag(). Tags can override this to
   * perform any custom processing.
   *
   * @param testContext   the current test context container
   */
  protected void doStartTag(TestContextContainer testContext) {
  }

  /**
   * Called when the closing tag is encountered.
   */
  public final int doEndTag() throws JspException {
    doEndTag(testContext);

    return EVAL_PAGE;
  }

  /**
   * Template method called during doEndTag(). Tags can override this to
   * perform any custom processing.
   *
   * @param testContext   the current test context container
   */
  protected void doEndTag(TestContextContainer testContext) {
  }

  /**
   * Called when an exception is thrown during the processing of this tag.
   *
   * @param t   the Throwable instance
   */
  public void doCatch(Throwable t) {
    this.throwable = t;
    throwable.printStackTrace();
    TestContext ctx = getTestContext();
    ctx.setStatus(TestContext.ERROR);
    ctx.setMessage(ExceptionUtils.getStackTraceAsString(throwable));
  }

  /**
   * Thrown after the invocation of the tag.
   */
  public void doFinally() {
    if (getTestContextManager() != null) {
      getTestContextManager().endTestContext(testContext);
    }
  }

  /**
   * Gets the name attribute (the name of the test/assertion being performed)
   *
   * @return  the name as a String
   */
  public String getName() {
    return name;
  }

  /**
   * Sets the name attribute (the name of the test/assertion being performed)
   *
   * @param name    the name of the test/assertion
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * Gets a reference to the throwable caught during tag
   * processing, from doCatch();
   *
   * @return    a Throwable instance
   */
  protected Throwable getThrowable() {
    return throwable;
  }

  /**
   * Sets a reference to the throwable caught during tag
   * processing, from doCatch();
   *
   * @param throwable   a Throwable instance
   */
  protected void setThrowable(Throwable throwable) {
    this.throwable = throwable;
  }

  /**
   * Gets the current TestContextManager instance that is managing the tests.
   *
   * @return    a TestContextManager instance
   */
  protected final TestContextManager getTestContextManager() {
    return (TestContextManager)pageContext.getRequest().getAttribute(Constants.TEST_CONTEXT_MANAGER);
  }

  /**
   * Gets a reference to the current tag test context, if applicable
   *
   * @return  a TagTestContext instance
   */
  protected final TagTestContext getTagTestContext() {
    return getTestContextManager().getTagTestContext();
  }

  /**
   * Gets a reference to the current test context.
   *
   * @return  a TestContext instance
   */
  protected final TestContextContainer getTestContext() {
    return this.testContext;
  }

}