/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.taginfo;

import java.io.Serializable;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * An in-memory representation of a tag attribute.
 *
 * @author    Simon Brown
 */
public final class TagAttributeInfo implements Serializable {

  /** the default type of attributes, if not specified */
  private static final String DEFAULT_TYPE = "java.lang.String";

  /** the name of the attribute */
  private String name;

  /** the type (class) of the attribute */
  private String type;

  /** a flag indicating whether the attribute is marked as required */
  private boolean required;

  /** a flag inidicating whether the attribute accepts request-time expressions */
  private boolean rtexprvalue;

  /**
   * Default, no args constructor.
   */
  public TagAttributeInfo() {
  }

  /**
   * Creates a new instance based upon the contents of an XML node.
   *
   * @param node    an XML (DOM) node
   */
  public TagAttributeInfo(Node node) {
    init(node);
  }

  /**
   * Helper method to initliase this instance.
   *
   * @param node    an XML (DOM) node
   */
  private void init(Node node) {

    NodeList children = node.getChildNodes();
    Node child;
    for (int i = 0; i < children.getLength(); i++) {
      child = children.item(i);
      String name = child.getNodeName();
      String value = child.getFirstChild().getNodeValue();

      if (name.equals("name")) {
        setName(value);
      } else if (name.equals("type")) {
        setType(value);
      } else if (name.equals("required")) {
        setRequired(value.equalsIgnoreCase("true") || value.equalsIgnoreCase("yes"));
      } else if (name.equals("rtexprvalue")) {
        setAcceptRequestTimeExpressions(value.equalsIgnoreCase("true") || value.equalsIgnoreCase("yes"));
      }
    }
  }

  /**
   * Gets the name of the attribute.
   *
   * @return  the name as a String
   */
  public String getName() {
    return this.name;
  }

  /**
   * Sets the name of the attribute.
   *
   * @param s   the name as a String
   */
  public void setName(String s) {
    this.name = s;
  }

  /**
   * Gets the type of the attribute, or the default if not specified.
   *
   * @return  the type as a String
   */
  public String getType() {
    // if not specified, return the default type, as stated by the JSP specification
    if (this.type == null) {
      return DEFAULT_TYPE;
    } else {
      return this.type;
    }
  }

  /**
   * Sets the type of the attribute.
   *
   * @param s   the fully qualified class name of the attribute
   */
  public void setType(String s) {
    this.type = s;
  }

  /**
   * Determines whether this attribute is marked as required.
   *
   * @return  true if the attribute is required, false otherwise
   */
  public boolean isRequired() {
    return this.required;
  }

  /**
   * Sets whether this attribute is marked as required.
   *
   * @param b   true if the attribute is required, false otherwise
   */
  public void setRequired(boolean b) {
    this.required = b;
  }

  /**
   * Determines whether this attribute is marked as accepting
   * runtime expressions (i.e. rtexprvalue=true).
   *
   * @return  true if the attribute is required, false otherwise
   */
  public boolean canAcceptRequestTimeExpressions() {
    return this.rtexprvalue;
  }

  /**
   * Sets whether this attribute is marked as accepting
   * runtime expressions (i.e. rtexprvalue=true).
   *
   * @param b   true if the attribute is required, false otherwise
   */
  public void setAcceptRequestTimeExpressions(boolean b) {
    this.rtexprvalue = b;
  }

}