/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.taginfo;

import java.io.Serializable;
import java.util.*;

/**
 * Represents information about a tag defined within a tag library.
 *
 * @author    Simon Brown
 */
public class TagInfo implements Serializable {

  /** the name of the tag */
  private String name;

  /** the name of the tag handler class */
  private String tagHandlerClass;

  /** the name of the TagExtraInfo class */
  private String tagExtraInfoClass;

  /** the body content of the tag */
  private String bodyContent;

  /** a short description of the tag */
  private String description;

  /** the collection of attributes that this tag can accept */
  private Map attributes = new HashMap();

  /** the collection of variable that this tag can introduce into the page */
  private Map variables = new HashMap();

  /**
   * Default, no args constructor.
   */
  public TagInfo() {
  }

  /**
   * Gets the name of this tag.
   *
   * @return    the name as a String
   */
  public String getName() {
    return this.name;
  }

  /**
   * Sets the name of this tag.
   *
   * @param s    the name as a String
   */
  public void setName(String s) {
    this.name = s;
  }

  /**
   * Gets the name of the tag handler class for this tag.
   *
   * @return    the fully qualified name of the tag handler class as a String
   */
  public String getTagHandlerClass() {
    return tagHandlerClass;
  }

  /**
   * Sets the name of the tag handler class.
   *
   * @param s   the fully qualified class name
   */
  public void setTagHandlerClass(String s) {
    this.tagHandlerClass = s;
  }

  /**
   * Gets the name of the TagExtraInfo class for this tag.
   *
   * @return    the fully qualified name of the TEI class as a String,
   *            or null if the tag doesn't use a TEI
   */
  public String getTagExtraInfoClass() {
    return this.tagExtraInfoClass;
  }

  /**
   * Sets the name of the TagExtraInfo class for this tag.
   *
   * @param s   the fully qualified class name as a String
   */
  public void setTagExtraInfoClass(String s) {
    this.tagExtraInfoClass = s;
  }

  /**
   * Gets the body content type for this tag. If a tag doesn't explicitly
   * define this, then the default is used.
   *
   * @return    the String descrinbing the body content
   */
  public String getBodyContent() {
    if (bodyContent != null) {
      return bodyContent;
    } else {
      return getDefaultBodyContent();
    }
  }

  /**
   * Gets the default body content type for this tag.
   *
   * @return  a String representing the default body content type
   */
  public String getDefaultBodyContent() {
    return "JSP";
  }

  /**
   * Sets the body content of this tag.
   *
   * @param s   the body content as a String
   */
  public void setBodyContent(String s) {
    this.bodyContent = s;
  }

  /**
   * Gets the description of this tag.
   *
   * @return    the description as a String
   */
  public String getDescription() {
    return description;
  }

  /**
   * Sets the description of this tag.
   *
   * @param s   the description of this tag as a String
   */
  public void setDescription(String s) {
    this.description = s;
  }

  /**
   * Adds an attribute definion to this tag.
   *
   * @param attr    a TagAttributeInfo object representing the attribute
   */
  public void add(TagAttributeInfo attr) {
    attributes.put(attr.getName(), attr);
  }

  /**
   * Adds a (scripting) variable definition to this tag.
   *
   * @param var   a TagVariableInfo object representing the variable
   */
  public void add(TagVariableInfo var) {
    variables.put(var.getName(), var);
  }

  /**
   * Gets information about a specific, named attribute.
   *
   * @param name    the name of the attribute
   * @return  a TagAttributeInfo instance representing the attribute
   */
  public TagAttributeInfo getAttribute(String name) {
    return (TagAttributeInfo)attributes.get(name);
  }

  /**
   * Gets a collection representing the attributes of this tag handler.
   *
   * @return  a Collection of TagAttributeInfo instances
   */
  public Collection getAttributes() {
    return Collections.unmodifiableCollection(attributes.values());
  }

  /**
   * Gets information about a specific, named variable.
   *
   * @param name    the name of the variable
   * @return  a TagVariableInfo instance representing the variable
   */
  public TagVariableInfo getVariable(String name) {
    return (TagVariableInfo)variables.get(name);
  }

  /**
   * Gets a collection representing the attributes of this tag handler.
   *
   * @return  a Collection of TagVariableInfo instances
   */
  public Collection getVariables() {
    return Collections.unmodifiableCollection(variables.values());
  }

  /**
   * An implementation of equals for this class.
   *
   * @param o   the Object to be tested against
   * @return  true if the object represents the same as this,
   *          false otherwise
   */
  public boolean equals(Object o) {
    if (o == this) {
      return true;
    }

    if (!(o instanceof TagInfo)) {
      return false;
    }

    TagInfo tagInfo = (TagInfo)o;
    return tagInfo.getName().equals(this.getName());
  }

}