/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.taginfo;

import java.io.Serializable;
import java.util.*;

/**
 * An im-memory representation of the header information from a tag library
 * descriptor file - the general information about the taglib.
 *
 * @author    Simon Brown
 */
public class TagLibraryInfo implements Serializable {

  /** the version of the taglib */
  private String version;

  /** the version of JSP required to support this taglib */
  private String jspVersion;

  /** the short name given to the taglib */
  private String shortName;

  /** the URI given to the taglib */
  private String uri;

  /** a short description of the taglib */
  private String description;

  /** the collection of tags that this TLD defines */
  protected Map tags = new TreeMap();

  /**
   * Default, no args constructor.
   */
  public TagLibraryInfo() {
  }

  /**
   * Gets the version of this tag library (as defined in the TLD).
   *
   * @return  the version as a String
   */
  public String getVersion() {
    return this.version;
  }

  /**
   * Sets the version of this tag library (as defined in the TLD).
   *
   * @param s   the version as a String
   */
  public void setVersion(String s) {
    this.version = s;
  }

  /**
   * Gets the JSP version required by this taglib.
   *
   * @return    the JSP version as a String
   */
  public String getJspVersion() {
    return jspVersion;
  }

  /**
   * Sets the JSP version required by the taglib.
   *
   * @param s   the JSP version as a String
   */
  public void setJspVersion(String s) {
    this.jspVersion = s;
  }

  /**
   * Gets the short name of this taglib.
   *
   * @return    the short name as a String
   */
  public String getShortName() {
    return shortName;
  }

  /**
   * Sets the short name of this taglib.
   *
   * @param s   the short name as a String
   */
  public void setShortName(String s) {
    this.shortName = s;
  }

  /**
   * Gets the URI for this taglib.
   *
   * @return    the URI as a String
   */
  public String getUri() {
    return uri;
  }

  /**
   * Sets the URI for this taglib.
   *
   * @param s   the URI as a String
   */
  public void setUri(String s) {
    this.uri = s;
  }

  /**
   * Gets the description of the taglib.
   *
   * @return    the description as a String
   */
  public String getDescription() {
    return description;
  }

  /**
   * Sets the description of the taglib.
   *
   * @param s   the description as a String
   */
  public void setDescription(String s) {
    this.description = s;
  }

  /**
   * Adds a TagInfo object (representing a tag) to this description of a
   * tag library.
   *
   * @param tag   a TagInfo instance representing the tag
   */
  public void add(TagInfo tag) throws DuplicateTagException {
    if (tags.containsKey(tag.getName())) {
      throw new DuplicateTagException("A tag called " + tag.getName() +
          " already exists within this tag library");
    }
    tags.put(tag.getName(), tag);
  }

  /**
   * Gets information for a specific, named tag.
   *
   * @param name    the name of the tag
   * @return  a TagInfo instance representing the tag
   */
  public TagInfo getTag(String name) {
    return (TagInfo)tags.get(name);
  }

  /**
   * Gets a collection of all tags in this tag library.
   *
   * @return  a Collection of TagInfo objects
   */
  public Collection getTags() {
    return Collections.unmodifiableCollection(tags.values());
  }

}