/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.test;

import java.util.Iterator;

import javax.servlet.jsp.PageContext;

import org.tagunit.*;
import org.tagunit.taginfo.TagInfo;

/**
 * Wraps up the automatic tests for a tag handler.
 *
 * @author    Simon Brown
 */
public class UserDefinedTagHandlerTestPackage extends AbstractTestPackage {

  /** information about the tag being tested */
  private TagInfo tagInfo;

  /** the base URI from which the resources are located */
  private String baseUri;

  /**
   * Creates a new test package for the specified tag.
   *
   * @param tagInfo   a TagInfo instance
   * @param baseUri   the base URI from which the tests can be located
   */
  public UserDefinedTagHandlerTestPackage(TagInfo tagInfo, String baseUri, PageContext pageContext) {
    super(pageContext);
    this.tagInfo = tagInfo;
    this.baseUri = baseUri;
  }

  /**
   * Runs the tests that are part of this test package.
   *
   * @param testContext   the TestContext to which these tests are to be run
   */
  public void executeTest(TestContextContainer testContext) {
    long start;
    long end;

    // find all test pages in the "baseUri/<tagName>" directory
    TestResources testResources = new TestResources(baseUri, getPageContext());
    if (testResources.getResources().isEmpty()) {
      System.out.println(" - Warning : No user defined tests for " + tagInfo.getName());
      TestContext ctx = new DefaultTestContext("User defined tag handler tests");
      ctx.setStatus(TestContext.WARNING);
      ctx.setMessage("Only automatic tests have been run against this tag");
      testContext.add(ctx);
      return;
    }

    System.out.println(" - Running user defined tests for " + tagInfo.getName());

    start = System.currentTimeMillis();

    // run all of the testXXX.jsp pages in the directory - if setUp.jsp
    // and tearDown.jsp exist, then these get run before and after each
    // testXXX.jsp page respectively
    String testUri;
    TestContextContainer includedTestContext;
    Iterator it = testResources.getResources().iterator();
    while (it.hasNext()) {
      testUri = (String)it.next();

      includedTestContext = new DefaultTestContextContainer("Inclusion of tests from " + testUri);
      //getTestContextManager().startTestContext(includedTestContext);
      testContext.add(includedTestContext);
      testContext.getManager().startTestContext(includedTestContext);

      try {
        // (1) run the setup process (optional)
        if (testResources.hasSetUp()) {
          System.out.println("   - Running setup");
          getPageContext().include(testResources.getSetUpUri());
        }

        // (2) perform the test
        System.out.println("   - Running " + testUri);
        getPageContext().include(testUri);

        // and now that the test has performed, run the
        // tear down process (optional)
        if (testResources.hasTearDown()) {
          System.out.println("   - Running teardown");
          getPageContext().include(testResources.getTearDownUri());
        }
      } catch (Exception e) {
        e.printStackTrace();
        includedTestContext.setStatus(TestContext.ERROR);
        includedTestContext.setMessage(e.getMessage());
      } finally {
        testContext.getManager().endTestContext(includedTestContext);
      }
    }

    end = System.currentTimeMillis();
    testContext.setTime(end - start);
    System.out.println(" (" + testContext.getTime() + " ms)");
  }


  /**
   * Gets the name of this test package.
   *
   * @return  the name of this test package as a String
   */
  public String getName() {
    return "TagUnit user defined tag handler tests";
  }

}