/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.taginfo;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import junit.framework.TestCase;

/**
 * Test cases for the TagLibraryInfoFactory class.
 *
 * @author      Simon Brown
 */
public class TagLibraryInfoFactoryTest extends TestCase {

  private static final String INVALID_TLD;
  private static final String TEST_TLD_11;
  private static final String TEST_TLD_12;

  static {
    StringBuffer buf = new StringBuffer();
    buf.append("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
    buf.append("<!DOCTYPE taglib");
    buf.append("  PUBLIC \"-//Sun Microsystems, Inc.//DTD JSP Tag Library 1.2//EN\"");
    buf.append("  \"http://java.sun.com/dtd/web-jsptaglibrary_1_2.dtd\">");

    buf.append("<taglib>");

    buf.append("  <tlib-version>1.0</tlib-version>");
    buf.append("  <jsp-version>1.2</jsp-version>");
    buf.append("  <short-name>aShortName</short-name>");
    buf.append("  <uri>/aUri</uri>");
    buf.append("  <description>aTagLibraryDescription</description>");

    buf.append("  <tag>");
    buf.append("    <name>aName</name>");
    buf.append("    <tag-class>aClass</tag-class>");
    buf.append("    <body-content>JSP</body-content>");
    buf.append("    <description>aTagDescription</description>");

    buf.append("</taglib>");

    INVALID_TLD = buf.toString();

    buf = new StringBuffer();
    buf.append("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
    buf.append("<!DOCTYPE taglib");
    buf.append("  PUBLIC \"-//Sun Microsystems, Inc.//DTD JSP Tag Library 1.1//EN\"");
    buf.append("  \"http://java.sun.com/j2ee/dtds/web-jsptaglibrary_1_1.dtd\">");

    buf.append("<taglib>");

    buf.append("  <tlibversion>1.0</tlibversion>");
    buf.append("  <jspversion>1.1</jspversion>");
    buf.append("  <shortname>aShortName</shortname>");
    buf.append("  <uri>/aUri</uri>");
    buf.append("  <info>aTagLibraryDescription</info>");

    buf.append("  <tag>");
    buf.append("    <name>aName</name>");
    buf.append("    <tagclass>aTagClass</tagclass>");
    buf.append("    <teiclass>aTeiClass</teiclass>");
    buf.append("    <bodycontent>JSP</bodycontent>");
    buf.append("    <info>aTagDescription</info>");

    buf.append("    <attribute>");
    buf.append("      <name>anAttribute</name>");
    buf.append("      <required>true</required>");
    buf.append("      <rtexprvalue>false</rtexprvalue>");
    buf.append("    </attribute>");

    buf.append("  </tag>");
    buf.append("</taglib>");

    TEST_TLD_11 = buf.toString();

    buf = new StringBuffer();
    buf.append("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
    buf.append("<!DOCTYPE taglib");
    buf.append("  PUBLIC \"-//Sun Microsystems, Inc.//DTD JSP Tag Library 1.2//EN\"");
    buf.append("  \"http://java.sun.com/dtd/web-jsptaglibrary_1_2.dtd\">");

    buf.append("<taglib>");

    buf.append("  <tlib-version>1.0</tlib-version>");
    buf.append("  <jsp-version>1.2</jsp-version>");
    buf.append("  <short-name>aShortName</short-name>");
    buf.append("  <uri>/aUri</uri>");
    buf.append("  <description>aTagLibraryDescription</description>");

    buf.append("  <tag>");
    buf.append("    <name>aName</name>");
    buf.append("    <tag-class>aTagClass</tag-class>");
    buf.append("    <tei-class>aTeiClass</tei-class>");
    buf.append("    <body-content>JSP</body-content>");
    buf.append("    <description>aTagDescription</description>");

    buf.append("    <variable>");
    buf.append("      <name-given>aVariable</name-given>");
    buf.append("      <variable-class>java.lang.String</variable-class>");
    buf.append("      <declare>true</declare>");
    buf.append("      <scope>AT_BEGIN</scope>");
    buf.append("    </variable>");

    buf.append("    <attribute>");
    buf.append("      <name>anAttribute</name>");
    buf.append("      <required>true</required>");
    buf.append("      <rtexprvalue>false</rtexprvalue>");
    buf.append("      <type>java.lang.String</type>");
    buf.append("    </attribute>");

    buf.append("  </tag>");
    buf.append("</taglib>");

    TEST_TLD_12 = buf.toString();
  }

  private TagLibraryInfoFactory factory;

  public void setUp() {
    factory = new TagLibraryInfoFactory();
  }

  /**
   * Tests what happens when a null InputStream is passed into the factory.
   */
  public void testNullInputStream() {
    try {
      factory.getTagLibraryInfo(null);
      fail();
    } catch (TagLibraryNotSupportedException e) {
      fail();
    } catch (TagLibraryInfoParseException e) {
      // caught this exception which is what should happen
    }
  }

  /**
   * Tests what happens when a null InputStream is passed into the factory.
   */
  public void testInvalidTLD() throws Exception {
    try {
      InputStream in = new ByteArrayInputStream(INVALID_TLD.getBytes());
      factory.getTagLibraryInfo(in);
      fail();
    } catch (TagLibraryNotSupportedException e) {
      fail();
    } catch (TagLibraryInfoParseException e) {
      // caught this exception which is what should happen
    }
  }

  /**
   * Tests that a JSP 1.1 TLD file can be parsed.
   */
  public void testJsp11TagLib() throws Exception {
    InputStream in = new ByteArrayInputStream(TEST_TLD_11.getBytes());
    TagLibraryInfo taglib = factory.getTagLibraryInfo(in);
    assertNotNull(taglib);
    assertEquals("1.0", taglib.getVersion());
    assertEquals("1.1", taglib.getJspVersion());
    assertEquals("aShortName", taglib.getShortName());
    assertEquals("/aUri", taglib.getUri());
    assertEquals("aTagLibraryDescription", taglib.getDescription());
    assertEquals(1, taglib.getTags().size());

    TagInfo tag = taglib.getTag("aName");
    assertNotNull(tag);
    assertEquals("aName", tag.getName());
    assertEquals("aTagClass", tag.getTagHandlerClass());
    assertEquals("aTeiClass", tag.getTagExtraInfoClass());
    assertEquals("JSP", tag.getBodyContent());
    assertEquals("aTagDescription", tag.getDescription());
    assertEquals(1, tag.getAttributes().size());
    assertEquals(0, tag.getVariables().size());

    TagAttributeInfo attr = tag.getAttribute("anAttribute");
    assertNotNull(attr);
    assertEquals("anAttribute", attr.getName());
    assertEquals("java.lang.String", attr.getType());
    assertTrue(attr.isRequired());
    assertFalse(attr.canAcceptRequestTimeExpressions());
  }

  /**
   * Tests that a JSP 1.2 TLD file can be parsed.
   */
  public void testJsp12TagLib() throws Exception {
    InputStream in = new ByteArrayInputStream(TEST_TLD_12.getBytes());
    TagLibraryInfo taglib = factory.getTagLibraryInfo(in);
    assertNotNull(taglib);
    assertEquals("1.0", taglib.getVersion());
    assertEquals("1.2", taglib.getJspVersion());
    assertEquals("aShortName", taglib.getShortName());
    assertEquals("/aUri", taglib.getUri());
    assertEquals("aTagLibraryDescription", taglib.getDescription());
    assertEquals(1, taglib.getTags().size());

    TagInfo tag = taglib.getTag("aName");
    assertNotNull(tag);
    assertEquals("aName", tag.getName());
    assertEquals("aTagClass", tag.getTagHandlerClass());
    assertEquals("aTeiClass", tag.getTagExtraInfoClass());
    assertEquals("JSP", tag.getBodyContent());
    assertEquals("aTagDescription", tag.getDescription());
    assertEquals(1, tag.getAttributes().size());
    assertEquals(1, tag.getVariables().size());

    TagVariableInfo var = tag.getVariable("aVariable");
    assertNotNull(var);
    assertEquals("aVariable", var.getName());
    assertEquals("java.lang.String", var.getType());

    TagAttributeInfo attr = tag.getAttribute("anAttribute");
    assertNotNull(attr);
    assertEquals("anAttribute", attr.getName());
    assertEquals("java.lang.String", attr.getType());
    assertTrue(attr.isRequired());
    assertFalse(attr.canAcceptRequestTimeExpressions());
  }

}